OPA: Open Plugin Architecture
=============================



## Root


`string` **name** *(required)* - Defines the name of the plugin

    "name": "hello.myplugin",

`string` **version** *(required)* - Defines the version of the plugin

    "version": "1.2.4",

`map` **requires** - Select dependencies with compatible versions. Versions are separated using spaces,
and each version token can consist of either `*` matching any (usually the latest) version, a version 
number such as `1.0` matching that exact version, or a operator and a version number. Valid operators
are `>`, `>=`, `==`, `<`, `<=`, `!=`, or `~`. All of these work as expected, exept the `~`: Using
`~1.0` is the same as `>=1.0 <2`, and `~1.0.0` the same as `>=1.0.0 <1.1`.

    "requires": {
        "hello.other": "1.0",
        "this.works.too": "~1.0.0",
        "combined": ">=1.0 <3.0",
    },

`array` **implements** - To "pair" the plugin to an application or API, the plugin SHOULD implement at least one interface defined by the host application. Interfaces are virtual, but should be picked in such a way to avoid future collisions. `com.foo.hello` is a good interface name, `app` isn't.

    "implements": [
        "com.foo.hello"
    ],

### Autoloaders, Stubs and Implementations

*Note: This is only relevant to PHP plugins (having `lang:'php'` or `lang` omitted.*

The autoloaders are defined using the same PSR style as in Composer manifests.
To autoload against the same directory as the plugin.json, specify an empty
path.

An autoloader can also be created when the instance is loaded, by setting the
`stub` autoloader to true and returning an implementation from the stub.

#### Method 1: Explicit autoloader with Stub

    ...
    "autoload": {
        "psr-4": { "My\\Namespace\\":"" }
    },
    "stub": "plugin.php",
    ...

#### Method 2: Implicit autoloader with Stub

The stub must return the plugin instance. The namespace of the instance will be
mapped to the path of the stub using a psr-4 autoloader.

    ...
    "autoload": {
        "stub":true
    },
    "stub": "plugin.php",
    ...

#### Method 3: Explicit autoloader with Implementation

    ...
    "autoload": {
        "psr-4": { "My\\Namespace\\":"" }
    },
    "implementation": "My\\Namespace\\MyPlugin",
    ...

#### Method 4: No autoloader with Stub

    ...
    "stub": "plugin.php",
    ...


## Distribution

### Directories

Plugins can be distributed in directories, or as archives intended to be extracted.
Every directory holding a plugin.json will be considered a plugin.

### Archives (zip)

The plugin.json file SHOULD be located in the root of the archive. An iplementation
COULD resolve a relative root in the archive, but this is not required for compliance
with this specification.

### Phar archives

As Phar only makes sense to PHP, PHP implementations SHOULD implement phar support.


## Repositories

OPA comes with a repository specification, OPR or Open Plugin Repository.

    {
        "repository": 1,        /* OPR version */
        "packages": { },        /* Package list */
        "include": [ ]          /* Include additional .json files with packages */
    }

Packages MUST be indexed by package name and version:
    ...
    "hello.myplugin": {
        "1.2.4": {
            "dist": "...",      /* URL to zip archive */
            "manifest": { }     /* The full manifest */
        }
    }


## FAQ

**Q: Is OPA limited to PHP?**

A: No, the `lang` property can be used to specify the plugin language. If a host is
capable of running plugins in multiple languages, or even has its own scripting
language, this can be used to filter out usable plugins.

**Q: Are OPA interfaces related to <language here> interfaces?**

A: No. In OPA lingo, an *interface* is considered more abstract. It is an application
or a related set of features in an application that the plugin can chose to implement.

