<?php

namespace NoccyLabs\Plugin\Loader;

class ArchiveLoader extends AbstractLoader
{
    protected $contents = [];

    public static function test($path)
    {
        if (!fnmatch("*.zip", $path, FNM_CASEFOLD)) {
            return false;
        }
        $zip = new \ZipArchive();
        $zip->open($path);
        if ($zip->statName("plugin.json") !== false) {
            return true;
        }
        return false;
    }

    public function createAutoloader($type,$ns,$path)
    {

        if (empty($this->contents)) {
            $zip = new \ZipArchive();
            $zip->open($this->root);
            $contents = [];
            for ($i = 0; $i < $zip->numFiles; $i++) {
                $contents[] = $zip->getNameIndex($i);
            }
            $this->contents = $contents;
        } else {
            $contents = $this->contents;
        }

        $ns = rtrim($ns,"\\")."\\";
        $path = rtrim($path,"/")."/";

        $loader = function ($class) use ($type, $ns, $path, $contents) {
            if (strncmp($ns, $class, strlen($ns)) !== 0)
                return;

            if ($type == 'psr-4') {
                $classRel = substr($class, strlen($ns));
                $classPath = str_replace("\\","/",$classRel).".php";
            } elseif ($type == 'psr-0') {
                $classPath = str_replace("\\","/",$class).".php";
            }

            $classPath = $path . $classPath;

            $srcPath = $this->getPath($classPath, true);
            if ($srcPath)
                require_once $srcPath;
        };
        return $loader;
    }

    public function getPath($path, $test=false)
    {
        $path = ltrim($path,"/");
        if ($test && (!in_array($path, $this->contents))) {
            return false;
        }
        return sprintf("zip://%s#%s", $this->root, $path);
    }

}

